
# q-expansion of newform 1143.2.a.c, downloaded from the LMFDB on 08 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1143
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [1], [-2], [4], [-3], [0], [-4], [3], [-5], [-5], [5], [-4], [-4], [-12], [1], [-5], [-5], [-8], [6], [-1], [8], [3], [-7], [4], [-3], [-12], [-8], [4], [6], [1], [18], [11], [-20], [-4], [24], [-14], [-1], [-16], [18], [26], [-14], [26], [-14], [-10], [11], [3], [-12], [20], [0], [-19], [27], [4], [0], [-17], [24], [-2], [29], [20], [-10], [-14], [6], [14], [-1], [4], [-30], [-18], [-14], [-9], [16], [-20], [8], [-17], [20], [6], [-34], [-26], [17], [-19], [18], [36], [8], [26], [2], [-14], [-24], [-6], [39], [18], [-7], [33], [6], [20], [-8], [-36], [-12], [-12], [0], [23], [-38], [-8], [-10], [-36], [-20], [-36], [-47], [-8], [-1], [5], [-12], [7], [-26], [7], [-30], [-20], [6], [12], [-45], [-6], [17], [-5], [11], [-22], [8], [10], [31], [-30], [-34], [50], [-19], [-47], [-15], [24], [-11], [27], [42], [2], [25], [26], [18], [17], [17], [12], [13], [-4], [51], [10], [-37], [44], [33], [-46], [27], [-19], [-33], [-17], [42], [25], [30], [22], [40], [-32], [60], [-50], [-50], [-42], [42], [58], [2], [35], [-18], [9], [38], [20], [27], [-32], [9], [-23], [54], [40], [30], [3], [13], [4], [-36], [14], [21], [-8], [-65], [-6], [-6], [-34], [-44], [20], [-8], [60], [36], [-22], [-49], [52], [-27], [15], [-3], [-13], [64], [63], [50], [-5], [12], [-54], [60], [-72], [-10], [2], [-20], [-56], [-26], [-16], [-73], [-48], [6], [-26], [13], [-39], [-24], [14], [-37], [-56], [-46], [-8], [61], [4], [59], [64], [-28], [-68], [-43], [-34], [-22], [4], [-35], [44], [-10], [33], [59], [-51], [-72], [-10], [-38], [12], [-44], [29], [-66], [64], [-34], [63], [36], [-30], [-51], [-10], [30], [-26], [-4], [30], [47], [-15], [34], [49], [38], [44], [-72], [10], [10], [-68], [-48], [20], [-46], [18], [-50], [-48], [47], [-35], [26], [-60], [-42], [2], [40], [-72], [-44], [-42], [-34], [86], [-30], [-34], [48], [-24], [66], [-76], [-68], [50], [-19], [25], [9], [-53], [-14], [10], [-14], [72], [-14], [-51], [70], [-62], [-72], [-6], [58], [-53], [60], [-63], [-30], [9], [34], [-77], [-58], [-31], [24], [62], [-55], [-4], [-8], [73], [-27], [-77], [53], [-2], [-4], [16], [-12], [12], [34], [-29], [-81], [40], [-16], [31], [-49], [-76], [-75], [-60], [44], [-69], [94], [-24], [-88], [81], [70], [15], [24], [24], [-33], [-88], [84], [48], [17], [66], [-55], [36], [51], [58], [24], [76], [52], [-64], [-45], [70], [5], [25], [21], [-37], [-78], [-50], [-46], [-82], [50], [-20], [-32], [-14], [15], [-49], [38], [-19], [-65], [99], [28], [92], [-84], [30], [-56], [11], [-14], [-33], [-54], [-31], [46], [-25], [65], [48], [-27], [-88], [-82], [103], [48], [-88], [74], [-16], [34], [-32], [9], [25], [38], [-32], [-11], [-20], [-32], [21], [12], [-18], [-76], [-48], [11], [28], [5]]
